from cbutil.ncu_keys import (
    dp_key,
    memory_bandwidth_key,
    memory_data_key,
    memory_read_bandwidth_key,
    memory_read_data_key,
    memory_write_bandwidth_key,
    memory_write_data_key,
    operational_intensity_key,
    p_max_key,
    runtime_key,
)
from dashboards.dashboard_base import (
    DashboardOptions,
    Repeat,
    build_dashboard,
    get_commit_annotation,
    get_grid_pos,
    pack_in_row,
)
from dashboards.influx_queries import join_variable_and
from dashboards.legends import Units
from dashboards.overrides import get_color_regex_override
from dashboards.panels import PanelInfos, get_time_series_panel
from dashboards.variables import (
    Filter,
    get_dashboard_variable,
    get_time_dependend_dashboard_variable,
)


def dashboard_uniformgridcpu():
    data_source = "InfluxDB-1"
    arch = "CPU"
    measurment_name = f"UniformGrid{arch}"
    row_repeat = "host"

    options = DashboardOptions(
        title=f"Uniform Grid {arch}",
        description=f"Benchmark dashboard for the Uniform Grid {arch} Benchmark from walberla",
        tags=[arch, "benchmark", "walberla", "Uniform Grid"],
        timezone="browser",
    )

    filters = [
        Filter(row_repeat),
        Filter("project_id", multi=True, default_value="walberla/walberla"),
        Filter("branch", multi=True, default_value="master"),
        Filter("collisionSetup"),
        Filter("mpi_num_processes"),
        Filter("streamingPattern"),
        Filter("timeStepStrategy"),
        Filter("stencil"),
        Filter("blocks_0"),
        Filter("blocks_1"),
        Filter("blocks_2"),
        Filter("cellsPerBlock_0"),
        Filter("cellsPerBlock_1"),
        Filter("cellsPerBlock_2"),
        Filter("periodic_0"),
        Filter("periodic_1"),
        Filter("periodic_2"),
    ]

    fields = [
        PanelInfos("Runtime (RDTSC) [s] STAT Max", Units.seconds),
        PanelInfos("mlupsPerProcess", Units.number),
        PanelInfos("DP [MFLOP/s] STAT Sum", Units.mflop_sec),
        PanelInfos("Operational intensity STAT Sum", Units.flop_per_byte),
        PanelInfos(
            'AVX DP [MFLOP/s] STAT Sum" / "DP [MFLOP/s] STAT Sum', Units.percent
        ),
        PanelInfos(
            "Memory bandwidth [MBytes/s] STAT Sum", Units.mbytes_per_second),
        PanelInfos(
            "Memory write bandwidth [MBytes/s] STAT Sum", Units.mbytes_per_second
        ),
        PanelInfos(
            "Memory read bandwidth [MBytes/s] STAT Sum", Units.mbytes_per_second
        ),
        PanelInfos("Memory data volume [GBytes] STAT Sum", Units.gigabyte),
        PanelInfos(
            "Memory read data volume [GBytes] STAT Sum", Units.gigabyte),
        PanelInfos(
            "Memory write data volume [GBytes] STAT Sum", Units.gigabyte),
        PanelInfos("Energy [J] STAT Sum", Units.joule),
        PanelInfos("Power [W] STAT Sum", Units.watt),
        PanelInfos("Clock [MHz] STAT Avg", Units.megahertz),
    ]

    filter_vars = [
        get_dashboard_variable(filter, measurment_name, data_source)
        for filter in filters[1:]
    ]
    filter_vars = [
        get_time_dependend_dashboard_variable(
            filters[0], measurment_name, data_source, inner_field_key="mlupsPerProcess"
        ),
        *filter_vars,
    ]

    row_repeat_var = [fv for fv in filter_vars if fv.name == row_repeat][0]

    where = join_variable_and([f.name for f in filters])
    annotations = get_commit_annotation(
        data_source, "red", "commits", measurment_name)

    panels = [
        get_time_series_panel(
            field,
            data_source,
            measurment_name,
            where=where,
            group_by=[f.name for f in filters],
        )
        for field in fields
    ]

    row = pack_in_row(
        title=f"{row_repeat}: ${row_repeat_var.name}",
        panels=[*panels],
        repeat=Repeat("v", row_repeat_var.name),
    )
    return build_dashboard(
        options, rows=[row], templating=[*filter_vars], annotations=annotations
    )


def dashboard_uniformgridgpu():
    data_source = "InfluxDB-1"
    arch = "GPU"
    measurment_name = f"UniformGrid{arch}"
    row_repeat = "host"

    options = DashboardOptions(
        title=f"Uniform Grid {arch}",
        description=f"Benchmark dashboard for the Uniform Grid {arch} Benchmark from walberla",
        tags=[arch, "benchmark", "walberla", "Uniform Grid"],
        timezone="browser",
    )

    filters = [
        Filter("host", multi=False, default_value="medusa"),
        Filter("project_id", multi=True, default_value="walberla/walberla"),
        Filter("branch", multi=True, default_value="master"),
        Filter("GPU"),
        Filter("collisionSetup"),
        Filter("mpi_num_processes"),
        Filter("streamingPattern"),
        Filter("timeStepStrategy"),
        Filter("stencil"),
        Filter("blocks_0"),
        Filter("blocks_1"),
        Filter("blocks_2"),
        Filter("cellsPerBlock_0"),
        Filter("cellsPerBlock_1"),
        Filter("cellsPerBlock_2"),
        Filter("gpuBlockSize_0"),
        Filter("gpuBlockSize_1"),
        Filter("gpuBlockSize_2"),
        Filter("periodic_0"),
        Filter("periodic_1"),
        Filter("periodic_2"),
    ]

    fields = [
        PanelInfos("mlupsPerProcess", Units.number),
    ]

    filter_vars = [
        get_dashboard_variable(filter, measurment_name, data_source)
        for filter in filters
    ]

    row_repeat_var = [fv for fv in filter_vars if fv.name == row_repeat][0]

    where = join_variable_and([f.name for f in filters])
    annotations = get_commit_annotation(
        data_source, "red", "commits", measurment_name)

    panels = [
        get_time_series_panel(
            field,
            data_source,
            measurment_name,
            where=where,
            group_by=[f.name for f in filters],
        )
        for field in fields
    ]

    row = pack_in_row(
        title=f"{row_repeat}: ${row_repeat_var.name}",
        panels=[*panels],
        repeat=Repeat("v", row_repeat_var.name),
    )
    return build_dashboard(
        options, rows=[row], templating=[*filter_vars], annotations=annotations
    )


def dashboard_uniformgridgpu_profile():
    data_source = "InfluxDB-1"
    arch = "GPU"
    measurment_name = f"UniformGrid{arch}_profile"
    row_repeat = "host"

    options = DashboardOptions(
        title=f"Uniform Grid {arch} GPU Profiling",
        description=f"Benchmark dashboard for the Uniform Grid {arch} Benchmark from walberla",
        tags=[arch, "benchmark", "walberla", "Uniform Grid", "profiling"],
        timezone="browser",
    )

    filters = [
        Filter("host", multi=False, default_value="medusa"),
        Filter("project_id", multi=True, default_value="walberla/walberla"),
        Filter("branch", multi=True, default_value="master"),
        Filter("GPU"),
        Filter("collisionSetup"),
        Filter("mpi_num_processes"),
        Filter("streamingPattern"),
        Filter("timeStepStrategy"),
        Filter("stencil"),
        Filter("blocks_0"),
        Filter("blocks_1"),
        Filter("blocks_2"),
        Filter("cellsPerBlock_0"),
        Filter("cellsPerBlock_1"),
        Filter("cellsPerBlock_2"),
        Filter("gpuBlockSize_0"),
        Filter("gpuBlockSize_1"),
        Filter("gpuBlockSize_2"),
        Filter("periodic_0"),
        Filter("periodic_1"),
        Filter("periodic_2"),
    ]

    fields = [
        PanelInfos(runtime_key, Units.seconds),
        PanelInfos(dp_key, Units.mflop_sec),
        PanelInfos(p_max_key, Units.mflop_sec),
        PanelInfos(f'{dp_key}"/"{p_max_key}', Units.percent),
        PanelInfos(operational_intensity_key, Units.flop_per_byte),
        PanelInfos(memory_bandwidth_key, Units.mbytes_per_second),
        PanelInfos(memory_write_bandwidth_key, Units.mbytes_per_second),
        PanelInfos(memory_read_bandwidth_key, Units.mbytes_per_second),
        PanelInfos(memory_data_key, Units.gigabyte),
        PanelInfos(memory_write_data_key, Units.gigabyte),
        PanelInfos(memory_read_data_key, Units.gigabyte),
    ]

    filter_vars = [
        get_dashboard_variable(filter, measurment_name, data_source)
        for filter in filters
    ]

    row_repeat_var = [fv for fv in filter_vars if fv.name == row_repeat][0]

    where = join_variable_and([f.name for f in filters])
    annotations = get_commit_annotation(
        data_source, "red", "commits", measurment_name)

    panels = [
        get_time_series_panel(
            field,
            data_source,
            measurment_name,
            where=where,
            group_by=[f.name for f in filters],
        )
        for field in fields
    ]

    row = pack_in_row(
        title=f"{row_repeat}: ${row_repeat_var.name}",
        panels=[*panels],
        repeat=Repeat("v", row_repeat_var.name),
    )
    return build_dashboard(
        options, rows=[row], templating=[*filter_vars], annotations=annotations
    )


def dashboard_fslbmgravitywave():
    data_source = "InfluxDB-1"
    arch = "CPU"
    measurment_name = "FSLBM_gravitywave"

    options = DashboardOptions(
        title="Free Surface LBM - Gravity Wave",
        description="Benchmark dashboard for the FSBLM Gravitywave Benchmark from walberla",
        tags=[arch, "benchmark", "walberla", "FSLBM"],
        timezone="browser",
    )

    filters = [
        Filter("host"),
        Filter("project_id", multi=True, default_value="walberla/walberla"),
        Filter("branch", multi=True, default_value="master"),
        Filter("numMPIProcesses"),
    ]

    fields = [
        PanelInfos("simulationTime", Units.seconds),
    ]

    filter_vars = [
        get_dashboard_variable(filter, measurment_name, data_source)
        for filter in filters
    ]

    where = join_variable_and([f.name for f in filters])
    annotations = get_commit_annotation(
        data_source, "red", "commits", measurment_name)
    groups = ["Communication", "Sweep", "MPI Barrier"]
    colors = ["red", "green", "blue"]
    overrides = [
        get_color_regex_override(f".*{group}:.*", color, mode="fixed")
        for group, color in zip(groups, colors)
    ]

    panels = [
        get_time_series_panel(
            field,
            data_source,
            measurment_name,
            where=where,
            group_by=[f.name for f in filters],
            gridPos=get_grid_pos(12, 24, 0, idx * (13)),
        )
        for idx, field in enumerate(fields)
    ]
    panels.append(
        get_time_series_panel(
            PanelInfos("/.*_percentage/", Units.percent),
            data_source,
            measurment_name,
            where=where,
            group_by=[f.name for f in filters],
            gridPos=get_grid_pos(12, 12, 0, len(fields) * (13)),
            drawStyle="bars",
            stacking={"mode": "normal"},
            fillOpacity=70,
            gradientMode="Opacity",
            tooltipMode="single",
            overrides=overrides,
            colorMode="fixed",
        )
    )

    panels.append(
        get_time_series_panel(
            PanelInfos("/Communication:.*_percentage/", Units.percent),
            data_source,
            measurment_name,
            where=where,
            group_by=[f.name for f in filters],
            gridPos=get_grid_pos(12, 12, 12, len(fields) * (13)),
            drawStyle="bars",
            stacking={"mode": "normal"},
            fillOpacity=70,
            gradientMode="Opacity",
            tooltipMode="single",
        )
    )

    panels.append(
        get_time_series_panel(
            PanelInfos(r"/Sweep:.*_percentage/", Units.percent),
            data_source,
            measurment_name,
            where=where,
            group_by=[f.name for f in filters],
            gridPos=get_grid_pos(12, 12, 0, (len(fields) + 1) * (13)),
            drawStyle="bars",
            stacking={"mode": "normal"},
            fillOpacity=70,
            gradientMode="Opacity",
            tooltipMode="single",
        )
    )

    panels.append(
        get_time_series_panel(
            PanelInfos(r"/MPI Barrier:.*_percentage/", Units.percent),
            data_source,
            measurment_name,
            where=where,
            group_by=[f.name for f in filters],
            gridPos=get_grid_pos(12, 12, 12, (len(fields) + 1) * (13)),
            drawStyle="bars",
            stacking={"mode": "normal"},
            fillOpacity=70,
            gradientMode="Opacity",
            tooltipMode="single",
        )
    )

    return build_dashboard(
        options, panels=panels, templating=filter_vars, annotations=annotations
    )
